"use strict";
var _OverwolfWindow_name, _OverwolfWindow_id;
Object.defineProperty(exports, "__esModule", { value: true });
exports.OverwolfWindowError = exports.OverwolfWindow = void 0;
const tslib_1 = require("tslib");
const NOT_CURRENT_WINDOW = 'Method does not support windows other than current';
class OverwolfWindow {
    constructor(name = null) {
        _OverwolfWindow_name.set(this, void 0);
        _OverwolfWindow_id.set(this, void 0);
        tslib_1.__classPrivateFieldSet(this, _OverwolfWindow_name, name, "f");
        tslib_1.__classPrivateFieldSet(this, _OverwolfWindow_id, null, "f");
    }
    get isCurrent() {
        return (tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_name, "f") === null);
    }
    obtain() {
        if (this.isCurrent) {
            return this.obtainCurrent();
        }
        return this.obtainByName();
    }
    async obtainCurrent() {
        const result = await new Promise(resolve => {
            overwolf.windows.getCurrentWindow(resolve);
        });
        if (result && result.success && result.window && result.window.id) {
            tslib_1.__classPrivateFieldSet(this, _OverwolfWindow_id, result.window.id, "f");
            return result.window;
        }
        tslib_1.__classPrivateFieldSet(this, _OverwolfWindow_id, null, "f");
        throw new OverwolfWindowError({ result });
    }
    async obtainByName() {
        const result = await new Promise(resolve => {
            overwolf.windows.obtainDeclaredWindow(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_name, "f"), resolve);
        });
        if (result && result.success && result.window && result.window.id) {
            tslib_1.__classPrivateFieldSet(this, _OverwolfWindow_id, result.window.id, "f");
            return result.window;
        }
        tslib_1.__classPrivateFieldSet(this, _OverwolfWindow_id, null, "f");
        throw new OverwolfWindowError({
            args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_name, "f")],
            result
        });
    }
    async restore() {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.restore(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
                result
            });
        }
    }
    async minimize() {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.minimize(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
                result
            });
        }
    }
    async maximize() {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.maximize(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
                result
            });
        }
    }
    async hide() {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.hide(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
                result
            });
        }
    }
    async changeSize(width, height, autoDpiResize = true) {
        await this.obtain();
        const changeSizeParams = {
            window_id: tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"),
            width,
            height,
            auto_dpi_resize: autoDpiResize
        };
        const result = await new Promise(resolve => {
            overwolf.windows.changeSize(changeSizeParams, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [changeSizeParams],
                result
            });
        }
    }
    async changePosition(left, top) {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.changePosition(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), left, top, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), left, top],
                result
            });
        }
    }
    async dragMove() {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.dragMove(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
        });
        if (result && result.success) {
            return result;
        }
        throw new OverwolfWindowError({
            args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
            result
        });
    }
    async dragResize(edge) {
        await this.obtain();
        overwolf.windows.dragResize(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), edge);
    }
    async getWindowState() {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.getWindowState(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
        });
        if (result && result.success) {
            return result;
        }
        throw new OverwolfWindowError({
            args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
            result
        });
    }
    async setTopmost(shouldBeTopmost) {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.setTopmost(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), shouldBeTopmost, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), shouldBeTopmost],
                result
            });
        }
    }
    async sendToBack() {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.sendToBack(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
                result
            });
        }
    }
    async bringToFront(grabFocus) {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.bringToFront(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), grabFocus, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), grabFocus],
                result
            });
        }
    }
    async setPosition(properties) {
        await this.obtain();
        const result = await new Promise(resolve => {
            overwolf.windows.setPosition(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), properties, resolve);
        });
        if (!result || !result.success) {
            throw new OverwolfWindowError({
                args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), properties],
                result
            });
        }
    }
    async isWindowVisibleToUser() {
        if (!this.isCurrent) {
            throw new Error(NOT_CURRENT_WINDOW);
        }
        const result = await new Promise(resolve => {
            overwolf.windows.isWindowVisibleToUser(resolve);
        });
        if (result && result.success) {
            return result;
        }
        throw new OverwolfWindowError({ result });
    }
    async close() {
        const win = await this.obtain();
        if (win && win.stateEx !== 'closed') {
            const result = await new Promise(resolve => {
                overwolf.windows.close(tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"), resolve);
            });
            if (!result || !result.success) {
                throw new OverwolfWindowError({
                    args: [tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f")],
                    result
                });
            }
        }
    }
    async toggle() {
        const win = await this.obtain();
        switch (win.stateEx) {
            case 'closed':
            case 'minimized':
            case 'hidden':
                await this.restore();
                break;
            default:
                await this.close();
                break;
        }
    }
    async setZoom(zoomFactor) {
        await this.obtain();
        overwolf.windows.setZoom(zoomFactor, tslib_1.__classPrivateFieldGet(this, _OverwolfWindow_id, "f"));
    }
    async center() {
        const [win, viewport] = await Promise.all([
            this.obtain(),
            OverwolfWindow.getViewportSize()
        ]);
        let left = (viewport.width / 2) - (win.width / 2), top = (viewport.height / 2) - (win.height / 2);
        left = Math.max(left, 0);
        top = Math.max(top, 0);
        left = Math.floor(left);
        top = Math.floor(top);
        await this.changePosition(left, top);
    }
    static getWindowsStates() {
        return new Promise(resolve => overwolf.windows.getWindowsStates(resolve));
    }
    static getViewportSize() {
        return new Promise((resolve, reject) => {
            overwolf.utils.getMonitorsList(result => {
                let width = -1, height = -1;
                for (const display of result.displays) {
                    if (display.is_primary) {
                        width = display.width;
                        height = display.height;
                    }
                }
                overwolf.games.getRunningGameInfo(game => {
                    if (game && game.isInFocus) {
                        width = game.logicalWidth;
                        height = game.logicalHeight;
                    }
                    if (width !== -1 && height !== -1) {
                        resolve({ width, height });
                    }
                    else {
                        reject(new Error('Could not get monitors width/height'));
                    }
                });
            });
        });
    }
    static getMonitorsList() {
        return new Promise(resolve => overwolf.utils.getMonitorsList(resolve));
    }
}
exports.OverwolfWindow = OverwolfWindow;
_OverwolfWindow_name = new WeakMap(), _OverwolfWindow_id = new WeakMap();
class OverwolfWindowError extends Error {
    constructor({ result, args = [] }) {
        let message = JSON.stringify(result);
        if (args && args.length) {
            message += ` args: ${JSON.stringify(args)}`;
        }
        super(message);
        this.name = 'OverwolfWindowError';
        const errConst = Error;
        if (errConst.captureStackTrace) {
            errConst.captureStackTrace(this, OverwolfWindowError);
        }
    }
}
exports.OverwolfWindowError = OverwolfWindowError;
//# sourceMappingURL=overwolf-window.js.map